module PostalCode

  VERSION = '0.4.0'

  CityOffset  = 0
  StateOffset = 1
  DB = File.join File.dirname(__FILE__), '..', 'db', 'US.tsv'
  FS = "\t"   # field separator
  RS = "\n"   # record separator

  @cache = {}

  #
  # Load the entire postal code database into memory.
  #
  def self.load_cache
    open(DB) do |db|
      db.each_line do |line|
        line.rstrip!  # remove RS
        pcode, city, state = line.split(FS)
        @cache[pcode] = [city, state]
      end
    end
  end

  #
  # Clear the in-memory postal code cache.
  #
  def self.clear_cache
    @cache.clear
  end

  #
  # Return the city for the given postal code.
  #
  def self.city postal_code
    fetch(postal_code)[CityOffset]
  end

  #
  # Return the state (two-letter abbreviation) for the given postal code.
  #
  def self.state postal_code
    fetch(postal_code)[StateOffset]
  end

  # 
  # Postal codes must be strings because:
  # 1. Postal codes are not calculable, therefore should not be numeric.
  # 2. A postal code with a leading zero would be an octal. However, *08880* is
  #    an invalid octal numeric, yet a valid postal code.
  #
  def self.valid_format? postal_code
    unless postal_code.is_a? String
      raise(TypeError, "postal code must be a string")
    end

    (postal_code =~ /^\d{5}(-\d{4})?$/) ? true : false
  end

  #
  # Fetch postal code record.
  #
  def self.fetch postal_code
    return [nil, nil] unless valid_format? postal_code
    pcode = postal_code[0..4] # use first 5 characters; ignore ZIP+4 extension

    if @cache.has_key? pcode
      @cache[pcode]
    else
      records = %x[grep "^#{pcode}#{FS}" #{DB}].split(RS)

      if records.empty?
        [nil, nil]
      else
        @cache[pcode] = records.last.split(FS)[1..2]
      end
    end
  end

  private_class_method :fetch

end
